#include "MPU_9DOF_objects.h"
#include "MPU_9DOF_resources.h"
#include "built_in.h"


// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

// Touch Panel module connections
sbit DriveX_Left at GPIOB_ODR.B1;
sbit DriveX_Right at GPIOB_ODR.B8;
sbit DriveY_Up at GPIOB_ODR.B9;
sbit DriveY_Down at GPIOB_ODR.B0;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
const ADC_THRESHOLD = 1500;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

int _object_count;
unsigned short object_pressed;
TLabel *local_label;
TLabel *exec_label;
int label_order;
TBox *local_box;
TBox *exec_box;
int box_order;


void Init_ADC() {
  ADC_Set_Input_Channel(_ADC_CHANNEL_8 | _ADC_CHANNEL_9);
  ADC1_Init();
  Delay_ms(100);
}
static void InitializeTouchPanel() {
  Init_ADC();
  TFT_Init_ILI9341_8bit(240, 320);

  TP_TFT_Init(240, 320, 8, 9);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen*  CurrentScreen;

  TScreen                Screen1;
  TBox                   Box3;
  TBox                   Box2;
  TBox                   Box1;
  TLabel                 Label1;
char Label1_Caption[9] = "MPU 9DOF";

  TLine                  Line1;
  TLabel                 Label9;
char Label9_Caption[17] = "mikroElektronika";

  TLabel                 Label10;
char Label10_Caption[15] = "www.mikroe.com";

  TBox                   Box4;
  TLabel                 Label5;
char Label5_Caption[6] = "Angle";

  TBox                   Box5;
  TLabel                 Label11;
char Label11_Caption[5] = "Gyro";

  TBox                   Box6;
  TLabel                 Label15;
char Label15_Caption[4] = "Mag";

  TBox                   Box7;
  TBox                   Box8;
  TLabel                 Label6;
char Label6_Caption[5] = "Temp";

  TLabel                 Label4;
char Label4_Caption[3] = "Z:";

  TLabel                 Label2;
char Label2_Caption[3] = "X:";

  TLabel                 Label3;
char Label3_Caption[3] = "Y:";

  TLabel                 accelX;
char accelX_Caption[5] = "Text";

  TLabel                 accelY;
char accelY_Caption[5] = "Text";

  TLabel                 accelZ;
char accelZ_Caption[5] = "Text";

  TLabel                 Label7;
char Label7_Caption[3] = "Z:";

  TLabel                 Label8;
char Label8_Caption[3] = "X:";

  TLabel                 Label12;
char Label12_Caption[3] = "Y:";

  TLabel                 gyroX;
char gyroX_Caption[16] = "Text";

  TLabel                 gyroY;
char gyroY_Caption[16] = "Text";

  TLabel                 gyroZ;
char gyroZ_Caption[16] = "Text";

  TLabel                 lblTemp;
char lblTemp_Caption[16] = "53.5 C";

  TLabel                 Label13;
char Label13_Caption[3] = "Z:";

  TLabel                 Label14;
char Label14_Caption[3] = "X:";

  TLabel                 Label16;
char Label16_Caption[3] = "Y:";

  TLabel                 magX;
char magX_Caption[16] = "Text";

  TLabel                 magY;
char magY_Caption[16] = "Text";

  TLabel                 magZ;
char magZ_Caption[16] = "Text";

  TLabel                 Label17;
char Label17_Caption[4] = "[]";

  TLabel                 Label18;
char Label18_Caption[8] = "[/sec]";

  TLabel                 Label19;
char Label19_Caption[4] = "raw";

  TLabel                 * const code Screen1_Labels[29]=
         {
         &Label1,              
         &Label9,              
         &Label10,             
         &Label5,              
         &Label11,             
         &Label15,             
         &Label6,              
         &Label4,              
         &Label2,              
         &Label3,              
         &accelX,              
         &accelY,              
         &accelZ,              
         &Label7,              
         &Label8,              
         &Label12,             
         &gyroX,               
         &gyroY,               
         &gyroZ,               
         &lblTemp,             
         &Label13,             
         &Label14,             
         &Label16,             
         &magX,                
         &magY,                
         &magZ,                
         &Label17,             
         &Label18,             
         &Label19              
         };
  TBox                   * const code Screen1_Boxes[8]=
         {
         &Box3,                
         &Box2,                
         &Box1,                
         &Box4,                
         &Box5,                
         &Box6,                
         &Box7,                
         &Box8                 
         };
  TLine                  * const code Screen1_Lines[1]=
         {
         &Line1                
         };



static void InitializeObjects() {
  Screen1.Color                     = 0xFFFF;
  Screen1.Width                     = 240;
  Screen1.Height                    = 320;
  Screen1.LabelsCount               = 29;
  Screen1.Labels                    = Screen1_Labels;
  Screen1.BoxesCount                = 8;
  Screen1.Boxes                     = Screen1_Boxes;
  Screen1.LinesCount                = 1;
  Screen1.Lines                     = Screen1_Lines;
  Screen1.ObjectsCount              = 38;


  Box3.OwnerScreen     = &Screen1;
  Box3.Order           = 0;
  Box3.Left            = 8;
  Box3.Top             = 190;
  Box3.Width           = 220;
  Box3.Height          = 45;
  Box3.Pen_Width       = 1;
  Box3.Pen_Color       = 0x0000;
  Box3.Visible         = 1;
  Box3.Active          = 0;
  Box3.Transparent     = 1;
  Box3.Gradient        = 0;
  Box3.Gradient_Orientation = 0;
  Box3.Gradient_Start_Color = 0xFFFF;
  Box3.Gradient_End_Color = 0xC618;
  Box3.Color           = 0xAEB8;
  Box3.PressColEnabled = 1;
  Box3.Press_Color     = 0xE71C;
  Box3.OnUpPtr         = 0;
  Box3.OnDownPtr       = 0;
  Box3.OnClickPtr      = 0;
  Box3.OnPressPtr      = 0;

  Box2.OwnerScreen     = &Screen1;
  Box2.Order           = 1;
  Box2.Left            = 8;
  Box2.Top             = 135;
  Box2.Width           = 220;
  Box2.Height          = 45;
  Box2.Pen_Width       = 1;
  Box2.Pen_Color       = 0x0000;
  Box2.Visible         = 1;
  Box2.Active          = 0;
  Box2.Transparent     = 1;
  Box2.Gradient        = 0;
  Box2.Gradient_Orientation = 0;
  Box2.Gradient_Start_Color = 0xFFFF;
  Box2.Gradient_End_Color = 0xC618;
  Box2.Color           = 0xAEB8;
  Box2.PressColEnabled = 1;
  Box2.Press_Color     = 0xE71C;
  Box2.OnUpPtr         = 0;
  Box2.OnDownPtr       = 0;
  Box2.OnClickPtr      = 0;
  Box2.OnPressPtr      = 0;

  Box1.OwnerScreen     = &Screen1;
  Box1.Order           = 2;
  Box1.Left            = 8;
  Box1.Top             = 79;
  Box1.Width           = 220;
  Box1.Height          = 45;
  Box1.Pen_Width       = 1;
  Box1.Pen_Color       = 0x0000;
  Box1.Visible         = 1;
  Box1.Active          = 0;
  Box1.Transparent     = 1;
  Box1.Gradient        = 0;
  Box1.Gradient_Orientation = 0;
  Box1.Gradient_Start_Color = 0xFFFF;
  Box1.Gradient_End_Color = 0xAEB8;
  Box1.Color           = 0xAEB8;
  Box1.PressColEnabled = 1;
  Box1.Press_Color     = 0xE71C;
  Box1.OnUpPtr         = 0;
  Box1.OnDownPtr       = 0;
  Box1.OnClickPtr      = 0;
  Box1.OnPressPtr      = 0;

  Label1.OwnerScreen     = &Screen1;
  Label1.Order           = 3;
  Label1.Left            = 44;
  Label1.Top             = 18;
  Label1.Width           = 161;
  Label1.Height          = 41;
  Label1.Visible         = 1;
  Label1.Active          = 1;
  Label1.Caption         = Label1_Caption;
  Label1.FontName        = Arial_Black27x38_Bold;
  Label1.Font_Color      = 0x0000;
  Label1.OnUpPtr         = 0;
  Label1.OnDownPtr       = 0;
  Label1.OnClickPtr      = 0;
  Label1.OnPressPtr      = 0;

  Line1.OwnerScreen     = &Screen1;
  Line1.Order           = 4;
  Line1.First_Point_X   = 6;
  Line1.First_Point_Y   = 300;
  Line1.Second_Point_X  = 230;
  Line1.Second_Point_Y  = 300;
  Line1.Visible         = 1;
  Line1.Pen_Width       = 1;
  Line1.Color           = 0x0000;

  Label9.OwnerScreen     = &Screen1;
  Label9.Order           = 5;
  Label9.Left            = 8;
  Label9.Top             = 304;
  Label9.Width           = 92;
  Label9.Height          = 14;
  Label9.Visible         = 1;
  Label9.Active          = 1;
  Label9.Caption         = Label9_Caption;
  Label9.FontName        = Arial11x14_Bold;
  Label9.Font_Color      = 0x0000;
  Label9.OnUpPtr         = 0;
  Label9.OnDownPtr       = 0;
  Label9.OnClickPtr      = 0;
  Label9.OnPressPtr      = 0;

  Label10.OwnerScreen     = &Screen1;
  Label10.Order           = 6;
  Label10.Left            = 137;
  Label10.Top             = 304;
  Label10.Width           = 93;
  Label10.Height          = 14;
  Label10.Visible         = 1;
  Label10.Active          = 1;
  Label10.Caption         = Label10_Caption;
  Label10.FontName        = Arial11x14_Bold;
  Label10.Font_Color      = 0xF800;
  Label10.OnUpPtr         = 0;
  Label10.OnDownPtr       = 0;
  Label10.OnClickPtr      = 0;
  Label10.OnPressPtr      = 0;

  Box4.OwnerScreen     = &Screen1;
  Box4.Order           = 7;
  Box4.Left            = 7;
  Box4.Top             = 84;
  Box4.Width           = 42;
  Box4.Height          = 18;
  Box4.Pen_Width       = 1;
  Box4.Pen_Color       = 0x0000;
  Box4.Visible         = 1;
  Box4.Active          = 0;
  Box4.Transparent     = 1;
  Box4.Gradient        = 0;
  Box4.Gradient_Orientation = 0;
  Box4.Gradient_Start_Color = 0xFFFF;
  Box4.Gradient_End_Color = 0xAEB8;
  Box4.Color           = 0x75D2;
  Box4.PressColEnabled = 1;
  Box4.Press_Color     = 0xE71C;
  Box4.OnUpPtr         = 0;
  Box4.OnDownPtr       = 0;
  Box4.OnClickPtr      = 0;
  Box4.OnPressPtr      = 0;

  Label5.OwnerScreen     = &Screen1;
  Label5.Order           = 8;
  Label5.Left            = 14;
  Label5.Top             = 86;
  Label5.Width           = 28;
  Label5.Height          = 15;
  Label5.Visible         = 1;
  Label5.Active          = 1;
  Label5.Caption         = Label5_Caption;
  Label5.FontName        = Tahoma11x13_Regular;
  Label5.Font_Color      = 0xFFFF;
  Label5.OnUpPtr         = 0;
  Label5.OnDownPtr       = 0;
  Label5.OnClickPtr      = 0;
  Label5.OnPressPtr      = 0;

  Box5.OwnerScreen     = &Screen1;
  Box5.Order           = 9;
  Box5.Left            = 7;
  Box5.Top             = 140;
  Box5.Width           = 42;
  Box5.Height          = 18;
  Box5.Pen_Width       = 1;
  Box5.Pen_Color       = 0x0000;
  Box5.Visible         = 1;
  Box5.Active          = 0;
  Box5.Transparent     = 1;
  Box5.Gradient        = 0;
  Box5.Gradient_Orientation = 0;
  Box5.Gradient_Start_Color = 0xFFFF;
  Box5.Gradient_End_Color = 0xAEB8;
  Box5.Color           = 0x75D2;
  Box5.PressColEnabled = 1;
  Box5.Press_Color     = 0xE71C;
  Box5.OnUpPtr         = 0;
  Box5.OnDownPtr       = 0;
  Box5.OnClickPtr      = 0;
  Box5.OnPressPtr      = 0;

  Label11.OwnerScreen     = &Screen1;
  Label11.Order           = 10;
  Label11.Left            = 16;
  Label11.Top             = 142;
  Label11.Width           = 23;
  Label11.Height          = 15;
  Label11.Visible         = 1;
  Label11.Active          = 1;
  Label11.Caption         = Label11_Caption;
  Label11.FontName        = Tahoma11x13_Regular;
  Label11.Font_Color      = 0xFFFF;
  Label11.OnUpPtr         = 0;
  Label11.OnDownPtr       = 0;
  Label11.OnClickPtr      = 0;
  Label11.OnPressPtr      = 0;

  Box6.OwnerScreen     = &Screen1;
  Box6.Order           = 11;
  Box6.Left            = 7;
  Box6.Top             = 195;
  Box6.Width           = 42;
  Box6.Height          = 18;
  Box6.Pen_Width       = 1;
  Box6.Pen_Color       = 0x0000;
  Box6.Visible         = 1;
  Box6.Active          = 0;
  Box6.Transparent     = 1;
  Box6.Gradient        = 0;
  Box6.Gradient_Orientation = 0;
  Box6.Gradient_Start_Color = 0xFFFF;
  Box6.Gradient_End_Color = 0x75D2;
  Box6.Color           = 0x75D2;
  Box6.PressColEnabled = 1;
  Box6.Press_Color     = 0xE71C;
  Box6.OnUpPtr         = 0;
  Box6.OnDownPtr       = 0;
  Box6.OnClickPtr      = 0;
  Box6.OnPressPtr      = 0;

  Label15.OwnerScreen     = &Screen1;
  Label15.Order           = 12;
  Label15.Left            = 17;
  Label15.Top             = 197;
  Label15.Width           = 21;
  Label15.Height          = 15;
  Label15.Visible         = 1;
  Label15.Active          = 1;
  Label15.Caption         = Label15_Caption;
  Label15.FontName        = Tahoma11x13_Regular;
  Label15.Font_Color      = 0xFFFF;
  Label15.OnUpPtr         = 0;
  Label15.OnDownPtr       = 0;
  Label15.OnClickPtr      = 0;
  Label15.OnPressPtr      = 0;

  Box7.OwnerScreen     = &Screen1;
  Box7.Order           = 13;
  Box7.Left            = 8;
  Box7.Top             = 245;
  Box7.Width           = 220;
  Box7.Height          = 45;
  Box7.Pen_Width       = 1;
  Box7.Pen_Color       = 0x0000;
  Box7.Visible         = 1;
  Box7.Active          = 0;
  Box7.Transparent     = 1;
  Box7.Gradient        = 0;
  Box7.Gradient_Orientation = 0;
  Box7.Gradient_Start_Color = 0xFFFF;
  Box7.Gradient_End_Color = 0xC618;
  Box7.Color           = 0xAEB8;
  Box7.PressColEnabled = 1;
  Box7.Press_Color     = 0xE71C;
  Box7.OnUpPtr         = 0;
  Box7.OnDownPtr       = 0;
  Box7.OnClickPtr      = 0;
  Box7.OnPressPtr      = 0;

  Box8.OwnerScreen     = &Screen1;
  Box8.Order           = 14;
  Box8.Left            = 7;
  Box8.Top             = 250;
  Box8.Width           = 42;
  Box8.Height          = 18;
  Box8.Pen_Width       = 1;
  Box8.Pen_Color       = 0x0000;
  Box8.Visible         = 1;
  Box8.Active          = 0;
  Box8.Transparent     = 1;
  Box8.Gradient        = 0;
  Box8.Gradient_Orientation = 0;
  Box8.Gradient_Start_Color = 0xFFFF;
  Box8.Gradient_End_Color = 0x75D2;
  Box8.Color           = 0x75D2;
  Box8.PressColEnabled = 1;
  Box8.Press_Color     = 0xE71C;
  Box8.OnUpPtr         = 0;
  Box8.OnDownPtr       = 0;
  Box8.OnClickPtr      = 0;
  Box8.OnPressPtr      = 0;

  Label6.OwnerScreen     = &Screen1;
  Label6.Order           = 15;
  Label6.Left            = 13;
  Label6.Top             = 251;
  Label6.Width           = 28;
  Label6.Height          = 15;
  Label6.Visible         = 1;
  Label6.Active          = 1;
  Label6.Caption         = Label6_Caption;
  Label6.FontName        = Tahoma11x13_Regular;
  Label6.Font_Color      = 0xFFFF;
  Label6.OnUpPtr         = 0;
  Label6.OnDownPtr       = 0;
  Label6.OnClickPtr      = 0;
  Label6.OnPressPtr      = 0;

  Label4.OwnerScreen     = &Screen1;
  Label4.Order           = 16;
  Label4.Left            = 184;
  Label4.Top             = 84;
  Label4.Width           = 10;
  Label4.Height          = 15;
  Label4.Visible         = 1;
  Label4.Active          = 1;
  Label4.Caption         = Label4_Caption;
  Label4.FontName        = Tahoma11x13_Regular;
  Label4.Font_Color      = 0x0000;
  Label4.OnUpPtr         = 0;
  Label4.OnDownPtr       = 0;
  Label4.OnClickPtr      = 0;
  Label4.OnPressPtr      = 0;

  Label2.OwnerScreen     = &Screen1;
  Label2.Order           = 17;
  Label2.Left            = 73;
  Label2.Top             = 84;
  Label2.Width           = 11;
  Label2.Height          = 15;
  Label2.Visible         = 1;
  Label2.Active          = 1;
  Label2.Caption         = Label2_Caption;
  Label2.FontName        = Tahoma11x13_Regular;
  Label2.Font_Color      = 0x0000;
  Label2.OnUpPtr         = 0;
  Label2.OnDownPtr       = 0;
  Label2.OnClickPtr      = 0;
  Label2.OnPressPtr      = 0;

  Label3.OwnerScreen     = &Screen1;
  Label3.Order           = 18;
  Label3.Left            = 130;
  Label3.Top             = 84;
  Label3.Width           = 11;
  Label3.Height          = 15;
  Label3.Visible         = 1;
  Label3.Active          = 1;
  Label3.Caption         = Label3_Caption;
  Label3.FontName        = Tahoma11x13_Regular;
  Label3.Font_Color      = 0x0000;
  Label3.OnUpPtr         = 0;
  Label3.OnDownPtr       = 0;
  Label3.OnClickPtr      = 0;
  Label3.OnPressPtr      = 0;

  accelX.OwnerScreen     = &Screen1;
  accelX.Order           = 19;
  accelX.Left            = 59;
  accelX.Top             = 99;
  accelX.Width           = 32;
  accelX.Height          = 21;
  accelX.Visible         = 1;
  accelX.Active          = 1;
  accelX.Caption         = accelX_Caption;
  accelX.FontName        = Tahoma16x19_Regular;
  accelX.Font_Color      = 0x0000;
  accelX.OnUpPtr         = 0;
  accelX.OnDownPtr       = 0;
  accelX.OnClickPtr      = 0;
  accelX.OnPressPtr      = 0;

  accelY.OwnerScreen     = &Screen1;
  accelY.Order           = 20;
  accelY.Left            = 116;
  accelY.Top             = 99;
  accelY.Width           = 32;
  accelY.Height          = 21;
  accelY.Visible         = 1;
  accelY.Active          = 1;
  accelY.Caption         = accelY_Caption;
  accelY.FontName        = Tahoma16x19_Regular;
  accelY.Font_Color      = 0x0000;
  accelY.OnUpPtr         = 0;
  accelY.OnDownPtr       = 0;
  accelY.OnClickPtr      = 0;
  accelY.OnPressPtr      = 0;

  accelZ.OwnerScreen     = &Screen1;
  accelZ.Order           = 21;
  accelZ.Left            = 170;
  accelZ.Top             = 99;
  accelZ.Width           = 32;
  accelZ.Height          = 21;
  accelZ.Visible         = 1;
  accelZ.Active          = 1;
  accelZ.Caption         = accelZ_Caption;
  accelZ.FontName        = Tahoma16x19_Regular;
  accelZ.Font_Color      = 0x0000;
  accelZ.OnUpPtr         = 0;
  accelZ.OnDownPtr       = 0;
  accelZ.OnClickPtr      = 0;
  accelZ.OnPressPtr      = 0;

  Label7.OwnerScreen     = &Screen1;
  Label7.Order           = 22;
  Label7.Left            = 182;
  Label7.Top             = 138;
  Label7.Width           = 10;
  Label7.Height          = 15;
  Label7.Visible         = 1;
  Label7.Active          = 1;
  Label7.Caption         = Label7_Caption;
  Label7.FontName        = Tahoma11x13_Regular;
  Label7.Font_Color      = 0x0000;
  Label7.OnUpPtr         = 0;
  Label7.OnDownPtr       = 0;
  Label7.OnClickPtr      = 0;
  Label7.OnPressPtr      = 0;

  Label8.OwnerScreen     = &Screen1;
  Label8.Order           = 23;
  Label8.Left            = 71;
  Label8.Top             = 139;
  Label8.Width           = 11;
  Label8.Height          = 15;
  Label8.Visible         = 1;
  Label8.Active          = 1;
  Label8.Caption         = Label8_Caption;
  Label8.FontName        = Tahoma11x13_Regular;
  Label8.Font_Color      = 0x0000;
  Label8.OnUpPtr         = 0;
  Label8.OnDownPtr       = 0;
  Label8.OnClickPtr      = 0;
  Label8.OnPressPtr      = 0;

  Label12.OwnerScreen     = &Screen1;
  Label12.Order           = 24;
  Label12.Left            = 128;
  Label12.Top             = 139;
  Label12.Width           = 11;
  Label12.Height          = 15;
  Label12.Visible         = 1;
  Label12.Active          = 1;
  Label12.Caption         = Label12_Caption;
  Label12.FontName        = Tahoma11x13_Regular;
  Label12.Font_Color      = 0x0000;
  Label12.OnUpPtr         = 0;
  Label12.OnDownPtr       = 0;
  Label12.OnClickPtr      = 0;
  Label12.OnPressPtr      = 0;

  gyroX.OwnerScreen     = &Screen1;
  gyroX.Order           = 25;
  gyroX.Left            = 57;
  gyroX.Top             = 153;
  gyroX.Width           = 32;
  gyroX.Height          = 21;
  gyroX.Visible         = 1;
  gyroX.Active          = 1;
  gyroX.Caption         = gyroX_Caption;
  gyroX.FontName        = Tahoma16x19_Regular;
  gyroX.Font_Color      = 0x0000;
  gyroX.OnUpPtr         = 0;
  gyroX.OnDownPtr       = 0;
  gyroX.OnClickPtr      = 0;
  gyroX.OnPressPtr      = 0;

  gyroY.OwnerScreen     = &Screen1;
  gyroY.Order           = 26;
  gyroY.Left            = 114;
  gyroY.Top             = 153;
  gyroY.Width           = 32;
  gyroY.Height          = 21;
  gyroY.Visible         = 1;
  gyroY.Active          = 1;
  gyroY.Caption         = gyroY_Caption;
  gyroY.FontName        = Tahoma16x19_Regular;
  gyroY.Font_Color      = 0x0000;
  gyroY.OnUpPtr         = 0;
  gyroY.OnDownPtr       = 0;
  gyroY.OnClickPtr      = 0;
  gyroY.OnPressPtr      = 0;

  gyroZ.OwnerScreen     = &Screen1;
  gyroZ.Order           = 27;
  gyroZ.Left            = 169;
  gyroZ.Top             = 153;
  gyroZ.Width           = 32;
  gyroZ.Height          = 21;
  gyroZ.Visible         = 1;
  gyroZ.Active          = 1;
  gyroZ.Caption         = gyroZ_Caption;
  gyroZ.FontName        = Tahoma16x19_Regular;
  gyroZ.Font_Color      = 0x0000;
  gyroZ.OnUpPtr         = 0;
  gyroZ.OnDownPtr       = 0;
  gyroZ.OnClickPtr      = 0;
  gyroZ.OnPressPtr      = 0;

  lblTemp.OwnerScreen     = &Screen1;
  lblTemp.Order           = 28;
  lblTemp.Left            = 89;
  lblTemp.Top             = 256;
  lblTemp.Width           = 65;
  lblTemp.Height          = 25;
  lblTemp.Visible         = 1;
  lblTemp.Active          = 1;
  lblTemp.Caption         = lblTemp_Caption;
  lblTemp.FontName        = Tahoma19x23_Regular;
  lblTemp.Font_Color      = 0x0000;
  lblTemp.OnUpPtr         = 0;
  lblTemp.OnDownPtr       = 0;
  lblTemp.OnClickPtr      = 0;
  lblTemp.OnPressPtr      = 0;

  Label13.OwnerScreen     = &Screen1;
  Label13.Order           = 29;
  Label13.Left            = 182;
  Label13.Top             = 194;
  Label13.Width           = 10;
  Label13.Height          = 15;
  Label13.Visible         = 1;
  Label13.Active          = 1;
  Label13.Caption         = Label13_Caption;
  Label13.FontName        = Tahoma11x13_Regular;
  Label13.Font_Color      = 0x0000;
  Label13.OnUpPtr         = 0;
  Label13.OnDownPtr       = 0;
  Label13.OnClickPtr      = 0;
  Label13.OnPressPtr      = 0;

  Label14.OwnerScreen     = &Screen1;
  Label14.Order           = 30;
  Label14.Left            = 71;
  Label14.Top             = 195;
  Label14.Width           = 11;
  Label14.Height          = 15;
  Label14.Visible         = 1;
  Label14.Active          = 1;
  Label14.Caption         = Label14_Caption;
  Label14.FontName        = Tahoma11x13_Regular;
  Label14.Font_Color      = 0x0000;
  Label14.OnUpPtr         = 0;
  Label14.OnDownPtr       = 0;
  Label14.OnClickPtr      = 0;
  Label14.OnPressPtr      = 0;

  Label16.OwnerScreen     = &Screen1;
  Label16.Order           = 31;
  Label16.Left            = 128;
  Label16.Top             = 195;
  Label16.Width           = 11;
  Label16.Height          = 15;
  Label16.Visible         = 1;
  Label16.Active          = 1;
  Label16.Caption         = Label16_Caption;
  Label16.FontName        = Tahoma11x13_Regular;
  Label16.Font_Color      = 0x0000;
  Label16.OnUpPtr         = 0;
  Label16.OnDownPtr       = 0;
  Label16.OnClickPtr      = 0;
  Label16.OnPressPtr      = 0;

  magX.OwnerScreen     = &Screen1;
  magX.Order           = 32;
  magX.Left            = 57;
  magX.Top             = 209;
  magX.Width           = 32;
  magX.Height          = 21;
  magX.Visible         = 1;
  magX.Active          = 1;
  magX.Caption         = magX_Caption;
  magX.FontName        = Tahoma16x19_Regular;
  magX.Font_Color      = 0x0000;
  magX.OnUpPtr         = 0;
  magX.OnDownPtr       = 0;
  magX.OnClickPtr      = 0;
  magX.OnPressPtr      = 0;

  magY.OwnerScreen     = &Screen1;
  magY.Order           = 33;
  magY.Left            = 114;
  magY.Top             = 209;
  magY.Width           = 32;
  magY.Height          = 21;
  magY.Visible         = 1;
  magY.Active          = 1;
  magY.Caption         = magY_Caption;
  magY.FontName        = Tahoma16x19_Regular;
  magY.Font_Color      = 0x0000;
  magY.OnUpPtr         = 0;
  magY.OnDownPtr       = 0;
  magY.OnClickPtr      = 0;
  magY.OnPressPtr      = 0;

  magZ.OwnerScreen     = &Screen1;
  magZ.Order           = 34;
  magZ.Left            = 169;
  magZ.Top             = 209;
  magZ.Width           = 32;
  magZ.Height          = 21;
  magZ.Visible         = 1;
  magZ.Active          = 1;
  magZ.Caption         = magZ_Caption;
  magZ.FontName        = Tahoma16x19_Regular;
  magZ.Font_Color      = 0x0000;
  magZ.OnUpPtr         = 0;
  magZ.OnDownPtr       = 0;
  magZ.OnClickPtr      = 0;
  magZ.OnPressPtr      = 0;

  Label17.OwnerScreen     = &Screen1;
  Label17.Order           = 35;
  Label17.Left            = 15;
  Label17.Top             = 106;
  Label17.Width           = 14;
  Label17.Height          = 15;
  Label17.Visible         = 1;
  Label17.Active          = 1;
  Label17.Caption         = Label17_Caption;
  Label17.FontName        = Tahoma11x13_Regular;
  Label17.Font_Color      = 0x0000;
  Label17.OnUpPtr         = 0;
  Label17.OnDownPtr       = 0;
  Label17.OnClickPtr      = 0;
  Label17.OnPressPtr      = 0;

  Label18.OwnerScreen     = &Screen1;
  Label18.Order           = 36;
  Label18.Left            = 12;
  Label18.Top             = 162;
  Label18.Width           = 35;
  Label18.Height          = 15;
  Label18.Visible         = 1;
  Label18.Active          = 1;
  Label18.Caption         = Label18_Caption;
  Label18.FontName        = Tahoma11x13_Regular;
  Label18.Font_Color      = 0x0000;
  Label18.OnUpPtr         = 0;
  Label18.OnDownPtr       = 0;
  Label18.OnClickPtr      = 0;
  Label18.OnPressPtr      = 0;

  Label19.OwnerScreen     = &Screen1;
  Label19.Order           = 37;
  Label19.Left            = 16;
  Label19.Top             = 216;
  Label19.Width           = 18;
  Label19.Height          = 15;
  Label19.Visible         = 1;
  Label19.Active          = 1;
  Label19.Caption         = Label19_Caption;
  Label19.FontName        = Tahoma11x13_Regular;
  Label19.Font_Color      = 0x0000;
  Label19.OnUpPtr         = 0;
  Label19.OnDownPtr       = 0;
  Label19.OnClickPtr      = 0;
  Label19.OnPressPtr      = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetBox(index)                 CurrentScreen->Boxes[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawLabel(TLabel *ALabel) {
  if (ALabel->Visible == 1) {
    TFT_Set_Font(ALabel->FontName, ALabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(ALabel->Caption, ALabel->Left, ALabel->Top);
  }
}

void DrawBox(TBox *ABox) {
  if (ABox->Visible == 1) {
    if (object_pressed == 1) {
      object_pressed = 0;
      TFT_Set_Brush(ABox->Transparent, ABox->Press_Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_End_Color, ABox->Gradient_Start_Color);
    }
    else {
      TFT_Set_Brush(ABox->Transparent, ABox->Color, ABox->Gradient, ABox->Gradient_Orientation, ABox->Gradient_Start_Color, ABox->Gradient_End_Color);
    }
    TFT_Set_Pen(ABox->Pen_Color, ABox->Pen_Width);
    TFT_Rectangle(ABox->Left, ABox->Top, ABox->Left + ABox->Width - 1, ABox->Top + ABox->Height - 1);
  }
}

void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

void DrawScreen(TScreen *aScreen) {
 unsigned int order;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short box_idx;
  TBox *local_box;
  unsigned short line_idx;
  TLine *local_line;
  char save_bled, save_bled_direction;

  object_pressed = 0;
  order = 0;
  label_idx = 0;
  box_idx = 0;
  line_idx = 0;
  CurrentScreen = aScreen;

  if ((display_width != CurrentScreen->Width) || (display_height != CurrentScreen->Height)) {
    save_bled = TFT_BLED;
    TFT_BLED           = 0;
    TFT_Init_ILI9341_8bit(CurrentScreen->Width, CurrentScreen->Height);
    TP_TFT_Init(CurrentScreen->Width, CurrentScreen->Height, 8, 9);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(ADC_THRESHOLD);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(CurrentScreen->Color);
    display_width = CurrentScreen->Width;
    display_height = CurrentScreen->Height;
    TFT_BLED           = save_bled;
  }
  else
    TFT_Fill_Screen(CurrentScreen->Color);


  while (order < CurrentScreen->ObjectsCount) {
    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (box_idx < CurrentScreen->BoxesCount) {
      local_box = GetBox(box_idx);
      if (order == local_box->Order) {
        box_idx++;
        order++;
        DrawBox(local_box);
      }
    }

    if (line_idx < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

  }
}

void Get_Object(unsigned int X, unsigned int Y) {
  label_order         = -1;
  box_order           = -1;
  //  Labels
  for ( _object_count = 0 ; _object_count < CurrentScreen->LabelsCount ; _object_count++ ) {
    local_label = GetLabel(_object_count);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  //  Boxes
  for ( _object_count = 0 ; _object_count < CurrentScreen->BoxesCount ; _object_count++ ) {
    local_box = GetBox(_object_count);
    if (local_box->Active == 1) {
      if (IsInsideObject(X, Y, local_box->Left, local_box->Top,
                         local_box->Width, local_box->Height) == 1) {
        box_order = local_box->Order;
        exec_box = local_box;
      }
    }
  }

  _object_count = -1;
  if (label_order >  _object_count )
    _object_count = label_order;
  if (box_order >  _object_count )
    _object_count = box_order;
}


void Process_TP_Press(unsigned int X, unsigned int Y) {
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnPressPtr != 0) {
          exec_box->OnPressPtr();
          return;
        }
      }
    }

  }
}

void Process_TP_Up(unsigned int X, unsigned int Y) {

  switch (PressedObjectType) {
    // Box
    case 6: {
      if (PressedObject != 0) {
        exec_box = (TBox*)PressedObject;
        if ((exec_box->PressColEnabled == 1) && (exec_box->OwnerScreen == CurrentScreen)) {
          DrawBox(exec_box);
        }
        break;
      }
      break;
    }
  }

  exec_label          = 0;

  Get_Object(X, Y);


  if (_object_count != -1) {
  // Labels
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Boxes
    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->OnUpPtr != 0)
          exec_box->OnUpPtr();
        if (PressedObject == (void *)exec_box)
          if (exec_box->OnClickPtr != 0)
            exec_box->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

void Process_TP_Down(unsigned int X, unsigned int Y) {

  object_pressed      = 0;
  exec_label          = 0;
  exec_box            = 0;

  Get_Object(X, Y);

  if (_object_count != -1) {
    if (_object_count == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (_object_count == box_order) {
      if (exec_box->Active == 1) {
        if (exec_box->PressColEnabled == 1) {
          object_pressed = 1;
          DrawBox(exec_box);
        }
        PressedObject = (void *)exec_box;
        PressedObjectType = 6;
        if (exec_box->OnDownPtr != 0) {
          exec_box->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Init_MCU() {
  GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_9);
  TFT_BLED = 1;
  TFT_Set_Default_Mode();
  TP_TFT_Set_Default_Mode();
}

void Start_TP() {
  Init_MCU();

  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(239, 3700, 224, 3679);    // Set calibration constants

  InitializeObjects();
  display_width = Screen1.Width;
  display_height = Screen1.Height;
  DrawScreen(&Screen1);
}
