/*
 * Project name:
     MPU_9DOF.vtft
 * Generated by:
     Visual TFT
 * Date of creation
     2014-12-04
 * Time of creation
     1:21:55 PM
 * Test configuration:
     MCU:             STM32F407VG
     Dev.Board:       mikromedia_for_STM32_M4_9A
     Oscillator:      140000000 Hz
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 */

#include "MPU_9DOF_objects.h"

// --- MPU 9DOF
#include "typedefs.h"
#include "MPU9150A_I2C.h"
#include "IMU_Processor.h"
#include "AHRS_Processor.h"

#define _AHRS_FILTER
//or 
//#define _IMU_FILTER

sbit MPU9150A_FSY at GPIOD_ODR.B4;
sbit MPU9150_INT at GPIOC_IDR.B2;

extern unsigned int tmrTicks;
extern tSensor MPU9150A;
char txt[32];

//Timer2: Actual Interrupt Time = 100 us
void InitTimer2(){
  RCC_APB1ENR.TIM2EN = 1;
  TIM2_CR1.CEN = 0;
  TIM2_PSC = 0;
  TIM2_ARR = 6000;
}

void Timer2_interrupt() iv IVT_INT_TIM2 {
  TIM2_SR.UIF = 0;
  tmrTicks++;
}

void Timer2_On(){
  NVIC_IntEnable(IVT_INT_TIM2);
  TIM2_DIER.UIE = 1;
  TIM2_CR1.CEN = 1;
}

void Timer2_Off() {
  NVIC_IntDisable(IVT_INT_TIM2);
  TIM2_DIER.UIE = 0;
  TIM2_CR1.CEN = 0;
}

void updateLabel(tlabel *lbl, char *str){
  TFT_Set_Font(lbl->Fontname, Box1.Color, FO_HORIZONTAL);
  TFT_Write_Text(lbl->Caption, lbl->Left, lbl->Top);
  strcpy(lbl->Caption, str);
  TFT_Set_Font(lbl->Fontname, lbl->Font_Color, FO_HORIZONTAL);
  DrawLabel(lbl);
}

void displayResults() {
  // Accel (degrees)
  IntToStr((int)(MPU9150A.out_accel.x * 90.0f),txt);
  updateLabel(&accelX, ltrim(txt));
  IntToStr((int)(MPU9150A.out_accel.y * 90.0f),txt);
  updateLabel(&accelY, ltrim(txt));
  IntToStr((int)(MPU9150A.out_accel.z * 90.0f),txt);
  updateLabel(&accelZ, ltrim(txt));
  // Gyro (degrees/second)
  IntToStr((int)(MPU9150A.out_gyro.x * 1000.0f / MPU9150A.gdt),txt);
  updateLabel(&gyroX, ltrim(txt));
  IntToStr((int)(MPU9150A.out_gyro.y * 1000.0f / MPU9150A.gdt),txt);
  updateLabel(&gyroY, ltrim(txt));
  IntToStr((int)(MPU9150A.out_gyro.z * 1000.0f / MPU9150A.gdt),txt);
  updateLabel(&gyroZ, ltrim(txt));
  // Mag
  WordToStr((int)(MPU9150A.mag.x),txt);
  updateLabel(&magX, ltrim(txt));
  IntToStr((int)(MPU9150A.mag.y),txt);
  updateLabel(&magY, ltrim(txt));
  IntToStr((int)(MPU9150A.mag.z),txt);
  updateLabel(&magZ, ltrim(txt));
  //temperature (celsius)
  sprintf(txt, "%.2f", MPU9150A.temp);
  strcat (txt, " C");
  updateLabel(&lblTemp, ltrim(txt));
}

// ---- MPU DOF
void main() {
  GPIO_Digital_Output (&GPIOD_BASE, _GPIO_PINMASK_4);
  GPIO_Digital_Input (&GPIOC_BASE, _GPIO_PINMASK_2);
  I2C1_Init_Advanced(400000, &_GPIO_MODULE_I2C1_PB67);
  MPU9150A_FSY = 0;
  Start_TP();
  MPU9150A_Init();
  MPU9150A_Detect();
  MAG_Detect();
  tmrTicks = 0;
  initTimer2();
  Timer2_On();
  MPU9150A_Read(); //initial read
  delay_ms(10);
  while (1) {
    Check_TP();
    MPU9150A_Read();
    #ifdef _IMU_FILTER
    IMU_Update();
    #else
    AHRS_Update();
    #endif
    displayResults();
    Delay_ms(100);
  }
}